////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Simple Communications Channel (SCP) Module
///
///     This file defines the SCP module API prototypes and definitions.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_pkt.h"

// ---- Constants and Types -------------------------------------------------

// Number of channels supported per SCP instance
#define TERA_SCP_NUM_CHAN     (8)

// Default MTU
#define TERA_SCP_DEFAULT_MTU  (1024)

// SCP instance handle
typedef void *TERA_SCP_HANDLE;

// --------------------------------------------------------------------------
//  fTERA_SCP_CHAN_OPEN_CBACK
///     SCP channel open callback
//
//  Description:
///     This function pointer is for a SCP channel open callback. When a SCP
///     channel is opened, i.e., it has gone through its handshaking and the
///     channel is ready for data transfers, then this callback is called to
///     notify the user.
///
typedef void (* fTERA_SCP_CHAN_OPEN_CBACK)(
    UINT8 chan_id,    ///< SCP channel ID
    void *user        ///< User supplied paramter
    );

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

extern INT32 tera_scp_init(
    struct pci_dev *dev
    );

extern void tera_scp_cleanup(
    struct pci_dev *dev
    );

extern INT32 tera_scp_create(
    struct pci_dev           *dev,
    UINT8                     active_chan,
    TERA_SCP_HANDLE          *handle,
    fTERA_SCP_CHAN_OPEN_CBACK open_cback,
    void                     *open_cback_arg
    );

extern void tera_scp_reset(
    TERA_SCP_HANDLE handle
    );

extern INT32 tera_scp_mtu_set(
    TERA_SCP_HANDLE handle,
    UINT16          mtu
    );

extern INT tera_scp_chan_is_ready(
    TERA_SCP_HANDLE handle,
    UINT8           chan_id
    );

extern void tera_scp_pkt_proc(
    struct pci_dev *dev,
    TERA_SCP_HANDLE handle,
    sTERA_MEM_BUF  *buf,
    UINT32          buf_len,
    UINT8          *chan_id,
    UINT8         **apdu,
    UINT32         *apdu_len,
    sTERA_PKT_Q    *pkt_q
    );

extern INT32 tera_scp_data_pkt_send(
    struct pci_dev *dev,
    TERA_SCP_HANDLE handle,
    UINT8           chan_id,
    sTERA_PKT_Q    *pkt_q,
    void           *buf,
    UINT32          buf_len
    );

