////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Configuration Data Class
///
///     This file defines the configuration data access class and related
///     structures.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include <wx/fileconf.h>
#include "tera_types.h"

// ---- Constants and Types -------------------------------------------------

// --------------------------------------------------------------------------
//  ePCOIP_AGENT_CUR_VISIBILE
///     Cursor visibility states
//
//  Description:
///     This enumeration defines the states of the local and host cursors.
///     When the local cursor and keyboard feature is enabled the user can
///     configure the visible state of the local (client drawn cursor) and
///     the remote (host drawn cursor) cursors. Each cursor can be visble
///     or hidden, with the restriction that at least one must be visible.
///
typedef enum
{
    PCOIP_AGENT_CUR_VISIBLE_BOTH      = 0,
    PCOIP_AGENT_CUR_VISIBLE_HOST      = 1,
    PCOIP_AGENT_CUR_VISIBLE_CLIENT    = 2,
    PCOIP_AGENT_CUR_VISIBLE_NUM
} ePCOIP_AGENT_CUR_VISIBILE;

// Registry location
#define PCOIP_AGENT_REG_KEY_COMPANY                 L"Teradici Corporation"
#define PCOIP_AGENT_REG_KEY_APPNAME                 L"pcoip_agent"

// [Features] section variables
#define PCOIP_AGENT_LOCAL_CFG_FEAT_SECTION          _T("/Features")
#define PCOIP_AGENT_GLOBAL_CFG_SECTION              _T("/")
#define PCOIP_AGENT_CFG_FEAT_LOCK_WS                _T("LockWorkstation")
#define PCOIP_AGENT_CFG_FEAT_LOCK_WS_INVALID        255l
#define PCOIP_AGENT_CFG_FEAT_LOCK_WS_DEFAULT        0l

// [Client specific features] section variables
#define PCOIP_AGENT_CFG_PORTAL_FEAT_SECTION         _T("/%S/Features")
#define PCOIP_AGENT_CFG_FEAT_LCUR_LKBD_EN           _T("LocalCursorKeyboardEnable")
#define PCOIP_AGENT_CFG_FEAT_LCUR_LKBD_EN_INVALID   255l
#define PCOIP_AGENT_CFG_FEAT_LCUR_LKBD_EN_DEFAULT   1l
#define PCOIP_AGENT_CFG_FEAT_LCUR_SHAPE             _T("LocalCursorShape")
#define PCOIP_AGENT_CFG_FEAT_LCUR_SHAPE_INVALID     255l
#define PCOIP_AGENT_CFG_FEAT_LCUR_SHAPE_DEFAULT     0l
#define PCOIP_AGENT_CFG_FEAT_LCUR_VIS               _T("LocalCursorVisibility")
#define PCOIP_AGENT_CFG_FEAT_LCUR_VIS_INVALID       255l
#define PCOIP_AGENT_CFG_FEAT_LCUR_VIS_DEFAULT       PCOIP_AGENT_CUR_VISIBLE_BOTH
#define PCOIP_AGENT_CFG_FEAT_LCUR_FADE              _T("LocalCursorFade")
#define PCOIP_AGENT_CFG_FEAT_LCUR_FADE_INVALID      255l
#define PCOIP_AGENT_CFG_FEAT_LCUR_FADE_DEFAULT      1l
#define PCOIP_AGENT_CFG_FEAT_HIDE_HCUR              _T("HideHostCursor")
#define PCOIP_AGENT_CFG_FEAT_HIDE_HCUR_INVALID      255l
#define PCOIP_AGENT_CFG_FEAT_HIDE_HCUR_DEFAULT      0l
#define PCOIP_AGENT_CFG_FEAT_LKBD_REPEAT_EN         _T("ClientKeyboardRepeatEnable")
#define PCOIP_AGENT_CFG_FEAT_LKBD_REPEAT_EN_INVALID 255l
#define PCOIP_AGENT_CFG_FEAT_LKBD_REPEAT_EN_DEFAULT 0l
#define PCOIP_AGENT_CFG_FEAT_LKBD_DELAY             _T("ClientKeyboardRepeatDelay")
#define PCOIP_AGENT_CFG_FEAT_LKBD_DELAY_INVALID     10000000l
#define PCOIP_AGENT_CFG_FEAT_LKBD_DELAY_DEFAULT     300l
#define PCOIP_AGENT_CFG_FEAT_LKBD_RATE              _T("ClientKeyboardRepeatRate")
#define PCOIP_AGENT_CFG_FEAT_LKBD_RATE_INVALID      10000000l
#define PCOIP_AGENT_CFG_FEAT_LKBD_RATE_DEFAULT      100

// [Monitors] section variables
#define PCOIP_AGENT_CFG_MON_SECTION                 _T("/%S/Monitors")
#define PCOIP_AGENT_CFG_MON_COUNT                   _T("MonitorCount")
#define PCOIP_AGENT_CFG_MON_COUNT_DEFAULT           0l
#define PCOIP_AGENT_CFG_SCREEN_COUNT                _T("ScreenCount")
#define PCOIP_AGENT_CFG_SCREEN_COUNT_DEFAULT        0l

// [Monitors/MonitorX] section variables
#define PCOIP_AGENT_CFG_MONX_SECTION                _T("/%S/Monitors/Monitor%u")
#define PCOIP_AGENT_CFG_MONX_VID                    _T("VendorID")
#define PCOIP_AGENT_CFG_MONX_VID_DEFAULT            wxEmptyString
#define PCOIP_AGENT_CFG_MONX_PID                    _T("ProductID")
#define PCOIP_AGENT_CFG_MONX_PID_DEFAULT            wxEmptyString
#define PCOIP_AGENT_CFG_MONX_SERIAL                 _T("SerialNumber")
#define PCOIP_AGENT_CFG_MONX_SERIAL_DEFAULT         wxEmptyString
#define PCOIP_AGENT_CFG_MONX_SIZE_X                 _T("SizeX")
#define PCOIP_AGENT_CFG_MONX_SIZE_X_DEFAULT         0l
#define PCOIP_AGENT_CFG_MONX_SIZE_Y                 _T("SizeY")
#define PCOIP_AGENT_CFG_MONX_SIZE_Y_DEFAULT         0l
#define PCOIP_AGENT_CFG_MONX_POS_X                  _T("PositionX")
#define PCOIP_AGENT_CFG_MONX_POS_X_DEFAULT          0l
#define PCOIP_AGENT_CFG_MONX_POS_Y                  _T("PositionY")
#define PCOIP_AGENT_CFG_MONX_POS_Y_DEFAULT          0l
#define PCOIP_AGENT_CFG_MONX_MON                    _T("Mon")
#define PCOIP_AGENT_CFG_MONX_MON_DEFAULT            255l

// --------------------------------------------------------------------------
//  cTERA_CFG
///     Configuration data
//
//  Description:
///     This class defines a configuration data object to allow reading and
///     writing configuration data. This class hides the details of storing
///     and retrieving the configuration data that determines how the
///     application is configured to operate.
///
class cTERA_CFG
{

public:

    // --- Public Functions -------------------------------------------------

    cTERA_CFG(void);

    ~cTERA_CFG(void);

    INT32 init(
        const wxString *global_filename,
        const wxString *local_filename);

    void set_ws_lock(
        bool lock_enable);

    bool get_ws_lock(void);

    void set_lcur_lkbd_en_setting(
        const char *client_mac,
        int lcur_lkbd_en);

    void get_lcur_lkbd_en_setting(
        const char *client_mac,
        int *lcur_lkbd_en);

    void set_lcur_vis_setting(
        const char *client_mac,
        int lcur_vis);

    void get_lcur_vis_setting(
        const char *client_mac,
        int *lcur_vis);

    void set_lcur_fade_setting(
        const char *client_mac,
        int lcur_fade);

    void get_lcur_fade_setting(
        const char *client_mac,
        int *lcur_fade);

    void set_repeat_en_setting(
        const char *client_mac,
        int lkbd_repeat_en);

    void get_repeat_en_setting(
        const char *client_mac,
        int *lkbd_repeat_en);

    void set_repeat_delay_setting(
        const char *client_mac,
        int lkbd_repeat_delay);

    void get_repeat_delay_setting(
        const char *client_mac,
        int *lkbd_repeat_delay);

    void set_repeat_rate_setting(
        const char *client_mac,
        int lkbd_repeat_rate);

    void get_repeat_rate_setting(
        const char *client_mac,
        int *lkbd_repeat_rate);

    INT32 delete_mon_data(
        const char *client_mac,
        int max_num_mon);

    void set_monitor_screen_counts(
        const char *client_mac,
        unsigned int monitor_count,
        unsigned int screen_count);

    void get_monitor_screen_counts(
        const char *client_mac,
        unsigned int *monitor_count,
        unsigned int *screen_count);

    void set_monitor_info(
        const char *client_mac,
        unsigned int monitor_num,
        wxString *vid,
        wxString *pid,
        wxString *serial,
        int size_x,
        int size_y,
        int pos_x,
        int pos_y,
        unsigned int monitor);

    void get_monitor_info(
        const char *client_mac,
        unsigned int monitor_num,
        wxString *vid,
        wxString *pid,
        wxString *serial,
        int *size_x,
        int *size_y,
        int *pos_x,
        int *pos_y,
        unsigned int *monitor);

protected:

    // --- Protected Functions ----------------------------------------------

    void get_setting_int(
        const wchar_t *setting_key,
        const wxString *local_path,
        const wxChar *global_path,
        int bad_local_val,
        int default_val,
        int *cfg_value);

    void get_setting_int(
        const wchar_t *setting_key,
        const wxChar *local_path,
        const wxChar *global_path,
        int bad_local_val,
        int default_val,
        int *cfg_value);

    // --- Protected Variables ----------------------------------------------

    wxFileConfig            *m_config;  ///< Application configuration

}; // cTERA_CFG

