////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     PCoIP Host Software Statistics Property Page Class
///
///     This file defines the PCoIP Host Software Statistics property page
///     class.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_types.h"

#include "tera_pcoip_agent_gui_res.h"

// ---- Constants and Types -------------------------------------------------

// --------------------------------------------------------------------------
//  cPCOIP_AGENT_GUI_PROP_STAT
///     PCoIP Host Software Statistics property page
//
//  Description:
///     This class defines the PCoIP Host Software Statistics property page.
///     This page shows the PCoIP session statistics if available.
///
class cPCOIP_AGENT_GUI_PROP_STAT : public cPCOIP_AGENT_GUI_PROP_STAT_BASE
{

public:

    // --- Public Functions -------------------------------------------------

    cPCOIP_AGENT_GUI_PROP_STAT(
        wxWindow *parent);

    ~cPCOIP_AGENT_GUI_PROP_STAT(void);

    void max_monitors_set(UINT32 max_monitors);

    // --- Public Variables -------------------------------------------------

protected:
    // --- Protected Functions ----------------------------------------------

    // Message handlers
    void evt_timer(
        wxTimerEvent& evt);

    void evt_reset_btn(
        wxCommandEvent& evt);

    void evt_init_dialog(
        wxInitDialogEvent& evt);

    void evt_close(
        wxCloseEvent& evt);

    // Function overrides

    // Local functions
    void prop_grid_populate(void);

    INT32 prop_grid_update(void);

    void prop_grid_reset(void);

    void help_btn_setup(int display_help);

    // --- Protected Variables ----------------------------------------------

    wxTimer m_update_timer;     ///< Heartbeat timer
    UINT32  m_bw_min;           ///< Minimum bandwidth usage
    UINT32  m_bw_max;           ///< Maximum bandwidth usage
    UINT64  m_last_tx_byte_cnt; ///< Last Tx byte count
    INT     m_get_stat_fail_cnt; ///< Counts the number of times
                                 ///< an error occurred reading
                                 ///< statistics from FW
    int     m_max_disp;          ///< Max number of displays

    DECLARE_EVENT_TABLE()
}; // cPCOIP_AGENT_GUI_PROP_STAT

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

